mod assembly;
mod display;
mod outline;

use nalgebra::DVector;
use rustc_hash::FxHashSet;
use slab::Slab;
use sycamore::prelude::*;

use assembly::{Assembly, Element};
use display::Display;
use outline::Outline;

#[derive(Clone)]
struct AppState {
    assembly: Assembly,
    selection: Signal<FxHashSet<usize>>
}

fn main() {
    sycamore::render(|| {
        let state = AppState {
            assembly: Assembly {
                elements: create_signal(Slab::new()),
                constraints: create_signal(Slab::new())
            },
            selection: create_signal(FxHashSet::default())
        };
        state.assembly.elements.update(
            |elts| elts.insert(
                Element {
                    id: String::from("wing_a"),
                    label: String::from("Wing A"),
                    color: [1.00_f32, 0.25_f32, 0.00_f32],
                    rep: DVector::<f64>::from_column_slice(&[0.5, 0.5, 0.0, 0.5, -0.25]),
                    constraints: {
                        let mut set = FxHashSet::default();
                        set.insert(1);
                        set.insert(2);
                        set
                    }
                }
            )
        );
        state.assembly.elements.update(
            |elts| elts.insert(
                Element {
                    id: String::from("wing_b"),
                    label: String::from("Wing B"),
                    color: [0.00_f32, 0.25_f32, 1.00_f32],
                    rep: DVector::<f64>::from_column_slice(&[-0.5, -0.5, 0.0, 0.5, -0.25]),
                    constraints: FxHashSet::default()
                },
            )
        );
        state.assembly.elements.update(
            |elts| elts.insert(
                Element {
                    id: String::from("central"),
                    label: String::from("Central"),
                    color: [0.75_f32, 0.75_f32, 0.75_f32],
                    rep: DVector::<f64>::from_column_slice(&[0.0, 0.0, 0.0, 0.4, -0.625]),
                    constraints: FxHashSet::default()
                }
            )
        );
        provide_context(state);
        
        view! {
            Outline {}
            Display {}
        }
    });
}